@@ -1,12 +1,13 @@
-use ansi_term::Color::{Green, Red};
+use ansi_term::ANSIGenericString;
 use anyhow::anyhow;
 use clap::{crate_authors, crate_description, crate_version, AppSettings, Clap};
 use filename_decoder::IDecoder;
+use lazy_static::lazy_static;
 use std::fs::File;
 use std::io::{BufReader, BufWriter};
 use std::vec;
-use zifu::{filename_decoder, FileNameEntry, ZIFURequirement};
-use zifu::{InputZIPArchive, ZipFileEncodingType};
+use zifu::InputZIPArchive;
+use zifu::{filename_decoder, FileNameEncodingType, FileNameEntry, FileNamesDiagnosis};
 
 #[derive(thiserror::Error, Debug)]
 enum InvalidArgument {
@@ -34,36 +35,41 @@ fn prepare_for_non_tty(style: ansi_term::Style) -> ansi_term::Style {
     }
 }
 
-fn zifu_requirement_to_color(requirement: &ZIFURequirement) -> ansi_term::Colour {
+/// Prints messages (`.get_status_primary_message()` & `.get_statius_note()`)
+pub fn print_status_message(diagnosis: &FileNamesDiagnosis) {
     use ansi_term::Colour::*;
-    use ZIFURequirement::*;
-    return match requirement {
-        NotRequired => Green,
-        MaybeRequired => Yellow,
-        Required => Red,
-    };
+    println!(
+        "{}  {}",
+        prepare_for_non_tty(
+            (if diagnosis.is_universal_archive() {
+                Green
+            } else {
+                Red
+            })
+            .bold()
+        )
+        .paint(diagnosis.get_status_primary_message()),
+        prepare_for_non_tty(
+            (if diagnosis.is_universal_archive() {
+                Green
+            } else {
+                Yellow
+            })
+            .bold()
+        )
+        .paint(diagnosis.get_status_note())
+    );
 }
 
-/// Prints messages (`.get_status_primary_message()` & `.get_statius_note()`)
-pub fn print_status_message(encoding_type: &ZipFileEncodingType) {
-    if let Some(note) = encoding_type.get_status_note() {
-        println!(
-            "{}  {}",
-            prepare_for_non_tty(
-                zifu_requirement_to_color(&encoding_type.is_zifu_required()).bold()
-            )
-            .paint(encoding_type.get_status_primary_message()),
-            prepare_for_non_tty(Green.bold()).paint(note)
-        );
-    } else {
-        println!(
-            "{}",
-            prepare_for_non_tty(
-                zifu_requirement_to_color(&encoding_type.is_zifu_required()).bold()
-            )
-            .paint((encoding_type.get_status_primary_message()).as_ref())
-        );
-    }
+fn print_you_do_not_have_to_apply_this_tool(diagnosis: &FileNamesDiagnosis) {
+    use ansi_term::Colour::*;
+    eprintln!(
+        "{}  {}\n{}  {}",
+        prepare_for_non_tty(Green.bold()).paint(diagnosis.get_status_primary_message()),
+        prepare_for_non_tty(Green.bold()).paint(diagnosis.get_status_note()),
+        prepare_for_non_tty(Green.bold()).paint("You do not have to apply this tool."),
+        prepare_for_non_tty(Yellow.bold()).paint("Existing.")
+    );
 }
 
 /// Decodes and prints file names in central directories to stdout
@@ -74,24 +80,29 @@ pub fn print_status_message(encoding_type: &ZipFileEncodingType) {
 /// * `utf8_decoder` - UTF-8 decoder (used when explicitly encoded in UTF-8)
 /// * `legacy_decoder` - Legacy charset decoder (used otherwise)
 fn list_names_in_archive(fie_name_entries: &[FileNameEntry], legacy_decoder: &dyn IDecoder) {
+    use ansi_term::Colour::*;
+    use FileNameEncodingType::*;
+    lazy_static! {
+        static ref REGULAR_UTF8: ANSIGenericString<'static, str> =
+            prepare_for_non_tty(Green.bold()).paint("REGULAR UTF-8");
+        static ref IRREGULAR_UTF8: ANSIGenericString<'static, str> =
+            prepare_for_non_tty(Red.bold()).paint("IRREGULAR UTF-8");
+        static ref ASCII_GREEN: ANSIGenericString<'static, str> =
+            prepare_for_non_tty(Green.bold()).paint("ASCII");
+        static ref GUESSED: ANSIGenericString<'static, str> =
+            prepare_for_non_tty(Red.bold()).paint("GUESSED");
+    }
     for entry in fie_name_entries {
-        if entry.is_encoding_explicit {
-            println!(
-                "{}:{}:{}",
-                Green.bold().paint("EXPLICIT"),
-                Green.bold().paint("UTF-8"),
+        match entry.encoding_type {
+            ExplicitRegularUTF8 => println!("{}:{}", &*REGULAR_UTF8, &entry.name),
+            ExplicitIrregularUTF8 => println!("{}:{}", &*IRREGULAR_UTF8, &entry.name),
+            ImplicitASCII => println!("{}:{}", &*ASCII_GREEN, &entry.name),
+            ImplicitNonASCII => println!(
+                "{} {}:{}",
+                prepare_for_non_tty(Red.bold()).paint(legacy_decoder.encoding_name()),
+                &*GUESSED,
                 &entry.name
-            );
-        } else {
-            println!(
-                "{}:{}:{}",
-                Red.bold().paint("GUESSED"),
-                legacy_decoder
-                    .color()
-                    .bold()
-                    .paint(legacy_decoder.encoding_name()),
-                &entry.name
-            );
+            ),
         }
     }
 }
@@ -158,6 +169,12 @@ struct CLIOptions {
     utf8: bool,
     #[clap(short, long, about = "Don't confirm")]
     yes: bool,
+    #[clap(
+        short,
+        long,
+        about = "Try to convert even if we don't have to apply this tool."
+    )]
+    force: bool,
 }
 
 impl CLIOptions {
@@ -179,7 +196,7 @@ fn main() -> anyhow::Result<()> {
     input_zip_file.check_unsupported_zip_type()?;
 
     if cli_options.check {
-        let archive_names_type = input_zip_file.check_file_name_encoding();
+        let archive_names_type = input_zip_file.diagnose_file_name_encoding();
         print_status_message(&archive_names_type);
         std::process::exit(if archive_names_type.is_universal_archive() {
             0
@@ -224,6 +241,14 @@ fn main() -> anyhow::Result<()> {
             &input_zip_file.get_file_names_list(guessed_encoder),
             guessed_encoder,
         );
+        if !cli_options.force
+            && input_zip_file
+                .diagnose_file_name_encoding()
+                .is_universal_archive()
+        {
+            print_you_do_not_have_to_apply_this_tool(&input_zip_file.diagnose_file_name_encoding());
+            std::process::exit(2);
+        }
 
         if behavior_flags.ask_user {
             eprint!("Are these file names correct? [Y/n]: ");
@@ -231,6 +256,13 @@ fn main() -> anyhow::Result<()> {
                 std::process::exit(1);
             }
         }
+    } else if !cli_options.force
+        && input_zip_file
+            .diagnose_file_name_encoding()
+            .is_universal_archive()
+    {
+        print_you_do_not_have_to_apply_this_tool(&input_zip_file.diagnose_file_name_encoding());
+        std::process::exit(2);
     }
     let output_zip_file_str = cli_options
         .output
@@ -279,6 +311,7 @@ mod tests {
         assert_eq!(cli_options.utf8, true);
         assert_eq!(cli_options.check, false);
         assert_eq!(cli_options.list, true);
+        assert_eq!(cli_options.force, false);
     }
 
     #[test]
@@ -303,6 +336,7 @@ mod tests {
         assert_eq!(cli_options.utf8, false);
         assert_eq!(cli_options.check, true);
         assert_eq!(cli_options.list, false);
+        assert_eq!(cli_options.force, false);
     }
 
     #[test]
@@ -326,5 +360,31 @@ mod tests {
         assert_eq!(cli_options.utf8, false);
         assert_eq!(cli_options.check, false);
         assert_eq!(cli_options.list, false);
+        assert_eq!(cli_options.force, false);
+    }
+
+    #[test]
+    fn extended_args_parse_test4() {
+        let cli_options = CLIOptions::parse_from(vec![
+            "zifu",
+            "before.zip",
+            "after.zip",
+            "--yes",
+            "-e",
+            "gbk",
+            "-f",
+        ]);
+        let global_flags = cli_options.to_behavior_flags();
+
+        assert_eq!(global_flags.ask_user, false);
+        assert_eq!(global_flags.verbose, true);
+
+        assert_eq!(cli_options.input, "before.zip");
+        assert_eq!(cli_options.output.as_deref(), Some("after.zip"));
+        assert_eq!(cli_options.encoding.as_deref(), Some("gbk"));
+        assert_eq!(cli_options.utf8, false);
+        assert_eq!(cli_options.check, false);
+        assert_eq!(cli_options.list, false);
+        assert_eq!(cli_options.force, true);
     }
 }
